/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.property.definitions;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.common.PropertyName;

import com.google.common.collect.Maps;

/**
 * The builder of property definition.
 */
public class PropertyDefinitionBuilder {

    private final Map<Class<? extends PropertyDefinition>, PropertyDefinition>
            augmentation = Maps.newHashMap();
    private PropertyDefinition.Access isReadOnly;
    private PropertyDefinition.IsRequired isRequired;
    private PropertyDefinitionKey key;
    private PropertyName propertyName;
    private PropertyDefinition.PropertyValueType propertyValueType;

    /**
     * Creates a default user role builder.
     */
    public PropertyDefinitionBuilder() {
    }

    /**
     * Creates a property definition builder from a property definition object.
     *
     * @param base the property definition object
     */
    public PropertyDefinitionBuilder(PropertyDefinition base) {
        if (base.getKey() == null) {
            key = new PropertyDefinitionKey(base.getPropertyName());
            propertyName = base.getPropertyName();
        } else {
            key = base.getKey();
            propertyName = key.getPropertyName();
        }
        isReadOnly = base.getIsReadOnly();
        isRequired = base.getIsRequired();
        propertyValueType = base.getPropertyValueType();
    }

    /**
     * Sets the property definition access to be used by the builder.
     *
     * @param readOnly property definition read-only value
     * @return self
     */
    public PropertyDefinitionBuilder isReadOnly(
    	PropertyDefinition.Access readOnly) {
        isReadOnly = readOnly;
        return this;
    }

    /**
     * Sets the property definition requirement to be used by the builder.
     *
     * @param required property definition
     * @return self
     */
    public PropertyDefinitionBuilder isRequired(
    	PropertyDefinition.IsRequired required) {
        isRequired = required;
        return this;
    }

    /**
     * Sets the property definition key to be used by the builder.
     *
     * @param key property definition key value
     * @return self
     */
    public PropertyDefinitionBuilder key(PropertyDefinitionKey key) {
        this.key = key;
        return this;
    }

    /**
     * Sets the property name to be used by the builder.
     *
     * @param name property name value
     * @return self
     */
    public PropertyDefinitionBuilder propertyName(PropertyName name) {
        propertyName = name;
        return this;
    }

    /**
     * Sets the property value type to be used by the builder.
     *
     * @param type property value type value
     * @return self
     */
    public PropertyDefinitionBuilder propertyValueType(
            PropertyDefinition.PropertyValueType type) {
        propertyValueType = type;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public PropertyDefinitionBuilder addAugmentation(
            Class<? extends PropertyDefinition> augmentationType,
            PropertyDefinition augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public PropertyDefinitionBuilder removeAugmentation(
            Class<?extends PropertyDefinition> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a property definition from this builder instance.
     *
     * @return property definition
     */
    public PropertyDefinition build() {
        return new PropertyDefinitionImpl(this);
    }

    /**
     * The implementation of property definition interface.
     */
    private static final class PropertyDefinitionImpl implements
            PropertyDefinition {

        private final Access isReadOnly;
        private final IsRequired isRequired;
        private final PropertyDefinitionKey key;
        private final PropertyName propertyName;
        private final PropertyValueType propertyValueType;
        private final Map<Class<? extends PropertyDefinition>,
                PropertyDefinition> augmentation;

        private PropertyDefinitionImpl(PropertyDefinitionBuilder base) {
            if (base.key == null) {
                key = new PropertyDefinitionKey(base.propertyName);
                propertyName = base.propertyName;
            } else {
                key = base.key;
                propertyName = key.getPropertyName();
            }
            isReadOnly = base.isReadOnly;
            isRequired = base.isRequired;
            propertyValueType = base.propertyValueType;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public Access getIsReadOnly() {
            return isReadOnly;
        }

        @Override
        public IsRequired getIsRequired() {
            return isRequired;
        }

        @Override
        public PropertyDefinitionKey getKey() {
            return key;
        }

        @Override
        public PropertyName getPropertyName() {
            return propertyName;
        }

        @Override
        public PropertyValueType getPropertyValueType() {
            return propertyValueType;
        }

        @Override
        public int hashCode() {
            return Objects.hash(isReadOnly, isRequired, key, propertyName,
                    propertyValueType, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            PropertyDefinitionImpl o = (PropertyDefinitionImpl) obj;
            return Objects.equals(isReadOnly, o.isReadOnly) &&
                    Objects.equals(isRequired, o.isRequired) &&
                    Objects.equals(key, o.key) &&
                    Objects.equals(propertyName, o.propertyName) &&
                    Objects.equals(propertyValueType, o.propertyValueType) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("isReadOnly", isReadOnly)
                    .add("isRequired", isRequired)
                    .add("key", key)
                    .add("propertyName", propertyName)
                    .add("propertyValueType", propertyValueType)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}