/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.action.definitions;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.common.ActionName;

/**
 * Representation of building an action definition.
 */
public class ActionDefinitionBuilder {

    private Map<Class<? extends ActionDefinition>, ActionDefinition>
            augmentation = new HashMap<>();
    private ActionName actionName;
    private ActionDefinitionKey key;
    private ActionDefinition.ParameterValueType pvType;

    /**
     * Creates a default action definition builder.
     */
    public ActionDefinitionBuilder() {
    }

    /**
     * Creates an action definition builder from a given action definition.
     *
     * @param base the action definition
     */
    public ActionDefinitionBuilder(ActionDefinition base) {
        if (base.getKey() == null) {
            key = new ActionDefinitionKey(base.getActionName());
            actionName = base.getActionName();
        } else {
            key = base.getKey();
            actionName = key.getActionName();
        }
        pvType = base.getParameterValueType();
        if (base instanceof ActionDefinitionImpl) {
            ActionDefinitionImpl impl = (ActionDefinitionImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the action name to be used by the builder.
     *
     * @param name the action name
     * @return self
     */
    public ActionDefinitionBuilder actionName(ActionName name) {
        actionName = name;
        return this;
    }

    /**
     * Sets the action definition key to be used by the builder.
     *
     * @param key action definition key
     * @return self
     */
    public ActionDefinitionBuilder key(ActionDefinitionKey key) {
        this.key = key;
        return this;
    }

    /**
     * Sets the parameter value type to be used by the builder.
     *
     * @param type parameter value type
     * @return self
     */
    public ActionDefinitionBuilder parameterValueType(
            ActionDefinition.ParameterValueType type) {
        pvType = type;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public ActionDefinitionBuilder addAugmentation(
            Class<? extends ActionDefinition> augmentationType,
            ActionDefinition augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public ActionDefinitionBuilder removeAugmentation(
            Class<? extends ActionDefinition> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds an action definition from this builder instance.
     *
     * @return action definition
     */
    public ActionDefinition build() {
        return new ActionDefinitionImpl(this);
    }

    /**
     * The implementation of action definition.
     */
    private static final class ActionDefinitionImpl
            implements ActionDefinition {

        private final ActionName actionName;
        private final ActionDefinitionKey key;
        private final ParameterValueType parameterValueType;
        private final Map<Class<? extends ActionDefinition>, ActionDefinition>
                augmentation;

        private ActionDefinitionImpl(ActionDefinitionBuilder base) {
            if (base.key == null) {
                key = new ActionDefinitionKey(base.actionName);
                actionName = base.actionName;
            } else {
                key = base.key;
                actionName = key.getActionName();
            }
            parameterValueType = base.pvType;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public ActionName getActionName() {
            return actionName;
        }

        @Override
        public ActionDefinitionKey getKey() {
            return key;
        }

        @Override
        public ParameterValueType getParameterValueType() {
            return parameterValueType;
        }

        @Override
        public int hashCode() {
            return Objects.hash(actionName, key, parameterValueType, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ActionDefinition other = (ActionDefinition) obj;
            if (!Objects.equals(actionName,
                    other.getActionName())) {
                return false;
            }
            if (!Objects.equals(key,
                    other.getKey())) {
                return false;
            }
            if (!Objects.equals(parameterValueType,
                    other.getParameterValueType())) {
                return false;
            }
            ActionDefinitionImpl otherImpl = (ActionDefinitionImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("actionName", actionName)
                    .add("key", key)
                    .add("parameterValueType", parameterValueType)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}