/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.user;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.user.roles.UserRole;

/**
 * The builder of user roles.
 */
public class UserRolesBuilder {

    private Map<UserRoles, UserRoles> augmentation = new HashMap<>();
    private List<UserRole> userRoles;

    /**
     * Creates a default user roles builder.
     */
    public UserRolesBuilder() {
    }

    /**
     * Creates a user roles builder from a user roles object.
     *
     * @param base the user roles object
     */
    public UserRolesBuilder(UserRoles base) {
        userRoles = base.getUserRoles();
    }

    /**
     * Sets the user role list to be used by the builder.
     *
     * @param value user role list
     * @return self
     */
    public UserRolesBuilder userRoles(List<UserRole> value) {
        userRoles = value;
        return this;
    }

    /**
     * Builds the user roles from this instance.
     *
     * @return user roles
     */
    public UserRoles build() {
        return new UserRolesImpl(this);
    }

    /**
     * The implementation of user role interface to create an instance.
     */
    private static final class UserRolesImpl implements UserRoles {

        private final List<UserRole> userRoles;
        private final Map<UserRoles, UserRoles> augmentation;

        private UserRolesImpl(UserRolesBuilder base) {
            userRoles = base.userRoles;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public List<UserRole> getUserRoles() {
            return userRoles;
        }

        @Override
        public int hashCode() {
            return Objects.hash(userRoles, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null){
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            UserRoles other = (UserRoles) obj;
            if (!Objects.equals(userRoles, other.getUserRoles())) {
                return false;
            }
            UserRolesImpl otherImpl = (UserRolesImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("user roles", userRoles)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}